#!/usr/bin/python

"""This file provides main functions to run the experiments:
using following parameters :
revisionToUse: hadoop revision to use; e.g. 0,2,4,8
leakConfigName: name of the leak configuration to use (w/o ".properties")
workloadSize: workload size (num iterations of kmeans)
resultDirName: the subdir to put the result files
profiling: [no]Profiling
isInstrumented: [instrumented]
"""


from subprocess import Popen, PIPE
from measure import *
from startup import *
from shutdown import *
from constants import *
from toolsAgingScripts import *


# Primary function of this file
def experiment(revisionToUse, leakConfigName, workloadSize, resultDirName, profiling, isInstrumented=""):
    """ main function of experiment.py"""

    print ("%s: starting scripted experiment ..." % current_time())

    clean_environment(revisionToUse)
    #prepare_output_directory(revisionToUse, leakConfigName, workloadSize, resultDirName, profiling, isInstrumented)
    outputList = prepare_output_directory(revisionToUse, leakConfigName,
                                          workloadSize, resultDirName, profiling, isInstrumented)

    #start hadoop
    start_up(revisionToUse, leakConfigName, profiling, isInstrumented, outputList)
    lom_snapshot(isInstrumented, 'beforeRun')
    #run k-means workload
    run_workload(revisionToUse, workloadSize, profiling, outputList)
    lom_snapshot(isInstrumented, 'afterRun')
    #stop hadoop
    shut_down(revisionToUse, profiling, outputList)
    collect_log_data(revisionToUse, isInstrumented, outputList)

    print ("%s: Experiment finished ..." % current_time())


def clean_environment(revisionToUse):
    """clean the environment from last runs"""
    processName = HADOOP_DIR + "/clean.sh"
    listCmdArgs = [revisionToUse]
    process = run_process(processName, listCmdArgs, HADOOP_DIR)
    print ("%s: ... cleaned environment ..." % current_time())
    return process


def prepare_output_directory(revisionToUse, leakConfigName, workloadSize, resultDirName, profiling, isInstrumented):
    """# prepare the result output directories"""

    resultBaseDir = join_paths(RESULT_DIR, resultDirName)
    resultMainDir = resultBaseDir + '/v' + revisionToUse + '-' + leakConfigName + '-' + workloadSize
    resultDataDir = resultMainDir + '/data'
    resultLogDir = resultMainDir + '/logs'
    expConfigFile = resultMainDir + '/exp.conf'
    profilingFile = join_paths(resultMainDir, isInstrumented) + 'Run.prof'
    outputList = [resultMainDir, profilingFile, resultDataDir, resultLogDir, expConfigFile]

    # run command mkdir for create resultMainDir directory
    make_dir(resultMainDir)
    if 'v' + isInstrumented == 'vinstrumented':
        make_dir(resultDataDir)
    make_dir(resultLogDir)
    f = open(outputList[4], 'a')
    f.write("Hadoop Revision: %s\n" % revisionToUse)
    f.write("LeakConf: %s\n" % leakConfigName)
    f.write("#KmeansIterations: %s\n" % workloadSize)
    f.write("Options: %s %s\n" % (profiling, isInstrumented))
    f.write("")
    f.close()
    print ("%s: ... prepared output directory ..." % current_time())
    return outputList


def start_up(revisionToUse, leakConfigName, profiling, isInstrumented, outputList):
    """ start Hadoop
    @param outputList: outputList[0]=resultMainDir, outputList[1]=profiling_file
    """
    os.chdir(BASE_DIR)
    if profiling == "Profiling":
        prof = open(outputList[1], 'a')
        prof.write("Startup:\n")
        prof.close()
        listCmdArgs = ['-av', '-o', outputList[1],
                       './startup.py', revisionToUse, leakConfigName,
                       outputList[0], '1', isInstrumented]
        processName = TIME_BIN
        process = run_process(processName, listCmdArgs, BASE_DIR)
        prof.close()
    else:
        startup(revisionToUse, leakConfigName, outputList[0], '1', isInstrumented)
    return


def run_workload(revisionToUse, workloadSize, profiling, outputList):
    """run the workload: k-mean from Mahout library
    @param outputList: outputList[4]=experiment_log_file, outputList[1]=profiling_file
    """
    os.chdir(MAHOUT_DIR)
    print
    print "#################################"
    f = open(outputList[4], 'a')
    f.write("Workload start date: %s\n" % current_time())
    if profiling == "Profiling":
        prof = open(outputList[1], 'a')
        prof.write("Workload:\n")
        prof.close()
        listCmdArgs = ['-av', '-o', outputList[1],
                       './workload1.sh', revisionToUse, workloadSize]
        processName = TIME_BIN
        process = run_process(processName, listCmdArgs, MAHOUT_DIR)

        # get total rss
        raw_rss = Popen(["ps", "--user", "felix", "-o", "rss,cmd"], stdout=PIPE)
        grep_hadoop = Popen(["grep", "hadoop"], stdin=raw_rss.stdout, stdout=PIPE)
        # Allows echo_rss to receive a SIGPIPE if grep_hadoop exits.
        raw_rss.stdout.close()
        output = grep_hadoop.communicate()
        # compute total rss
        totalRss = compute_total_rss(output)
        prof = open(outputList[1], 'a')
        prof.write("\nTotalRSS: %s\n\n" % totalRss)
        prof.close()
    else:
        listCmdArgs = [revisionToUse, workloadSize]
        processName = MAHOUT_DIR + '/workload1.sh'
        process = run_process(processName, listCmdArgs, MAHOUT_DIR)
    f.write("Workload end date: %s\n" % current_time())
    f.close()
    print "#################################"
    print ("%s : ... applied the workload ..." % current_time())
    return process


def shut_down(revisionToUse, profiling, outputList):
    """stop the experiment, stop Hadoop
    @param outputList: outputList[0]=resultMainDir, outputList[1]=profiling_file
    """
    os.chdir(BASE_DIR)
    if profiling == "Profiling":
        prof = open(outputList[1], 'a')
        prof.write("Shutdown:\n")
        prof.close()
        listCmdArgs = ['-av', '-o', outputList[1],
                       './shutdown.py', revisionToUse, outputList[0]]
        processName = TIME_BIN
        process = run_process(processName, listCmdArgs, BASE_DIR)
    else:
        shutdown(revisionToUse, outputList[0])
    return


def collect_log_data(revisionToUse, isInstrumented, outputList):
    """collect log files and result data
    @param outputList: outputList[2]=resultDataDir, outputList[3]=resultLogDir
    """
    revisionToUseDir = join_paths(HADOOP_DIR, revisionToUse)
    primaryLogDir = revisionToUseDir + '/logs'
    listSourceDirs = [revisionToUseDir, primaryLogDir, MAHOUT_DIR]
    if 'v' + isInstrumented == 'vinstrumented':
        move_multi_files(revisionToUseDir, outputList[2], extension='*.csv')
    for item in listSourceDirs:
        move_multi_files(item, outputList[3], extension='*.log')


if __name__ == "__main__":
    # check parameter amount
    if total_cmd_args() < 5:
        print ("Missing arguments: <revision> <leakConf> <workloadSize> "
               "<result-subdir> <[no]Profiling> [instrumented]")
        exit()
    # retrieve parameters
    fileName, restArgs = sys.argv[0], sys.argv[1:]
    if total_cmd_args() == 5:
        revisionToUse, leakConfigName, workloadSize, resultDirName, profiling = restArgs
        isInstrumented = ""
    else:
        revisionToUse, leakConfigName, workloadSize, resultDirName, profiling, isInstrumented = restArgs
    experiment(revisionToUse, leakConfigName, workloadSize, resultDirName, profiling, isInstrumented)
